import os
import time

import requests

SUCCESS = 'SUCCESS'
FAILURE = 'FAILURE'
PROCESSING = 'PROCESSING'
ERROR = 'ERROR'


class MessagesMixin:

    @property
    def upload_successful(self):
        return '\nFile uploaded successfully'

    @property
    def retry_msg(self):
        return (
            'You submission is being evaluated.'
            '\n\tNote:If your submission is in evaluation status for'
            ' more than 30 min please contact support with submission'
            ' id, on a PRIVATE chat.\n\tPLEASE DO NOT SHARE YOUR API'
            ' KEY ON A PUBLIC THREAD')

    @property
    def error_msg(self):
        return (
            'Oops something went wrong during transgalactic transmission.'
            'Are you sure about the submission id?')

    @property
    def max_submission(self):
        return (
            'You have exceeded maximum allowed submissions'
            '\nPlease proceed with marking your final submission')


class Result(MessagesMixin):

    def __init__(self, json):
        self._json = json
        self._keys = set()
        self.submission_id = None
        self.status = None
        self.message = None
        self.score = None
        self._setattr()

    def __call__(self, json):
        self._json = json
        self._setattr()

    def _setattr(self):
        self._clear()
        for k, v in self._json.items():
            setattr(self, k, v)
            self._keys.add(k)

    def _clear(self):
        for key in self._keys:
            delattr(self, key)
        self._keys = set()

    def print_upload_msg(self):
        text = ('\tYou submission id is {}'
                '\n\tPlease take a note of the submission id for'
                ' finalizing your submission.')
        print(self.upload_successful)
        print(text.format(self.submission_id))

    def print_result_msg(self):
        text = '\tThe submission accuracy is {}'.format(self.score)
        print(text)

    def print_result_failure_msg(self):
        text = '\tThe submission accuracy is {}'.format(self.score)
        print(text)

    def print_timeout_msg(self):
        text = ('Evaluation is taking longer that usual.'
                '\nPlease wait for 10 minutes and try: '
                '\n\t\tmlr.result({})'.format(self.submission_id))
        print(text)

    def print_list_submissions(self):
        print('Your submissions are:')
        for submission_id, score in self.result.items():
            print('\t{}\t\t{}'.format(submission_id, score))

    def print_finalize_submission(self):
        text = ('Submission ({}) with a score of {}'.format(
            self.submission_id, self.score) +
                ' is marked for final evaluation')
        print(text)

    @staticmethod
    def handle_401(other_self):
        text = (
            'Request Unauthorized!'
            '\nPlease verify your api key and employee id.'
            '''Employee id:{0.participant_id}
            API Key{0.api_key}'''.format(other_self))
        print(text)

    @staticmethod
    def handle_429():
        text = (
            '\tYou have exceeded maximum allowed request.'
            '\n\tPlease proceed with finalizing your submission')
        print(text)

    def handle_500(self, action):
        text = 'An error occurred during {}'.format(action)
        if action == 'uploading file':
            text += '\tHint: Please check the file format.'
        print(text)


class MLRank:
    """
    import mlrank
    mlr = mlrank('participant_id', 'your_api_key')
    mlr.score('file_path/file.csv')
    mlr.submit('submission_id')
    """
    __version__ = '0.0.1a1.post1'

    def __init__(self, participant_id: int, api_key: str,
                 service_description_uri: str = None, **kwargs):
        self.participant_id = participant_id
        self.api_key = api_key
        self._base_url = self._get_remote_service(
            service_description_uri)
        self._wait_sec = kwargs.get('wait', 5)

    @staticmethod
    def _get_remote_service(uri: str) -> dict:
        url = uri if uri else os.environ.get('SERVICE_DESCRIPTION_URI')
        if url is None:
            raise ValueError('Please provide evaluation service uri.')
        response = requests.get(url)
        if response.ok:
            return response.json().get('uri')
        return dict()

    def _network_issue(self, status_code, action):
        if status_code == 401:
            Result.handle_401(self)
        elif status_code == 429:
            Result.handle_429()
        elif status_code == 500:
            Result.handle_500()
        else:
            raise ConnectionError(
                'An error occurred while {}'
                ': status code {}'.format(action, status_code))

    def _file_upload(self, file: str) -> Result:
        if not os.path.isfile(file):
            raise FileNotFoundError('{} does not exist.'.format(file))
        if not isinstance(self.participant_id, int):
            raise ValueError('{0.participant_id} is not  valid,'.format(self) +
                             ' pass an integer value.')
        headers = {
            'x-api-key': self.api_key,
            'Content-Type': 'application/octet-stream',
        }

        with open(file, 'rb') as data:
            url = '{0._base_url}/upload/{0.participant_id}/data.csv'.format(self)
            response = requests.put(url, data=data, headers=headers)
            if response.ok:
                return Result(response.json())
            self._network_issue(response.status_code, 'uploading file')

    def _get_score(self, submission_id: str) -> Result:
        headers = {
            'x-api-key': self.api_key,
        }
        url = '{}/result/{}/{}'.format(
            self._base_url, self.participant_id, submission_id)
        response = requests.get(url, headers=headers)
        if response.ok:
            return Result(response.json())
        self._network_issue(response.status_code, 'retrieving result')

    def _await_eval(self, retry: bool = False):
        waiting = '\nWaiting for evaluation'
        retrying = '\nEvaluation pending'
        print(retrying if retry else waiting,
              end='', flush=True)
        for _ in range(self._wait_sec):
            time.sleep(1)
            print('.', end='', flush=True)
        print()

    @staticmethod
    def _print_result(result: Result):
        if result.status == SUCCESS:
            result.print_result_msg()
        elif result.status == PROCESSING:
            print(result.retry_msg)
        elif result.status == ERROR:
            print(result.error_msg)

    def result(self, submission_id: str):
        result = self._get_score(submission_id)
        if result:
            self._print_result(result)

    def submit(self, file: str) -> None:
        print('Preparing to upload file...')
        result = self._file_upload(file)
        if result:
            result.print_upload_msg()
            self._await_eval()
            print('\nRetrieving result...')
            result = self._get_score(result.submission_id)
            if result:
                for _ in range(3):
                    if result is None or result.status != PROCESSING:
                        break
                    self._await_eval(retry=True)
                    result = self._get_score(result.submission_id)
                else:
                    result.print_timeout_msg()
                    return
                if result:
                    self._print_result(result)

    def _list_submissions(self) -> Result:
        headers = {
            'x-api-key': self.api_key,
        }
        url = '{}/list/{}'.format(self._base_url, self.participant_id)
        response = requests.get(url, headers=headers)
        if response.ok:
            return Result(response.json())
        self._network_issue(response.status_code, 'listing submission')

    def list_submissions(self):
        result = self._list_submissions()
        if result:
            result.print_list_submissions()

    def _mark_submission(self, submission_id):
        headers = {
            'x-api-key': self.api_key,
        }
        url = '{}/submit/{}/{}'.format(
            self._base_url, self.participant_id, submission_id)
        response = requests.post(url, headers=headers)
        if response.ok:
            return Result(response.json())
        self._network_issue(response.status_code, 'finalizing submission')

    def mark_final(self, submission_id):
        result = self._mark_submission(submission_id)
        if result:
            result.print_finalize_submission()



import pickle
import pandas as pd


def type_cast(val):
    try:
        return float(val)
    except (ValueError, TypeError):
        return val.strip().lower()


class Litmus:

    def __init__(self, dataframe=None):
        self.df = dataframe
        self._tc = self._load_tc()
        self._eda = self._load_eda()

    def __call__(self, file, eda):
        score = 0
        try:
            dataframe = pd.read_csv(file)
            self.df = dataframe
            score += self._test()
            dataframe = pd.read_csv(eda)
            self.df = dataframe
            score += self._test_eda()
        except Exception as ex:
            print(ex)
        print(score)

    @staticmethod
    def _load_tc():
        import os
        directory, filename = os.path.split(__file__)
        return pickle.load(open(os.path.join(directory, 'tc.dat'), 'rb'))

    @staticmethod
    def _load_eda():
        import os
        directory, filename = os.path.split(__file__)
        return pickle.load(open(os.path.join(directory, 'eda.dat'), 'rb'))

    def _test_cn(self):
        error_score = 0
        cn = self._tc.get('cn')
        for col in cn:
            if col not in self.df.columns:
                error_score += 20
        return error_score

    def _test_rc(self):
        error_score = 0
        rc = self._tc.get('rc')
        rec_cnt = self.df.shape[0]
        if rec_cnt != rc[0]:
            error_score += (abs(rec_cnt - rc[0]) / rc[0]) * 100
        non_na_rec_cnt = self.df.dropna().shape[0]
        if non_na_rec_cnt != rc[1]:
            error_score += (abs(non_na_rec_cnt - rc[1]) / rc[1]) * 100
        return error_score

    def _test_col(self):
        error_score = 0
        cols = self._tc.get('col')
        for col in cols:
            name, has_null, non_na_cnt, unique = col
            if name in self.df.columns:
                if has_null != bool(self.df[name].isnull().sum()):
                    error_score += 1
                if non_na_cnt != self.df[name].dropna().shape[0]:
                    error_score += 1
                if unique != self.df[name].unique().shape[0]:
                    error_score += 1
            else:
                error_score += 5
        return error_score

    def _test(self):
        max_error = 450
        error_score = 0
        error_score += self._test_cn()
        error_score += self._test_rc()
        error_score += self._test_col()
        error_score = int(round(error_score))
        score = max_error - error_score
        return (score/max_error) * 30

    def _test_eda(self):
        max_score = 11
        error = 0
        lst = list()
        for item in self.df.ans.tolist():
            lst.append(type_cast(item))
        for x, y in zip(self._eda, lst):
            if x != y:
                error += 1
        return ((max_score-error)/max_score) * 70


litmus = Litmus()



#!/usr/bin/env python

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mlrank",
    version="0.0.1a1.post1",
    author="Prabakaran Kumaresshan",
    author_email="k_prabakaran@hotmail.com",
    description="Python client to submit output to evaluation server.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/nixphix/mlrank",
    packages=setuptools.find_packages(exclude=['docs', 'tests*']),
    install_requires=[
        'requests >= 2.20.0',
        'pandas >= 0.22.0'
    ],
    python_requires='>=3.5',
    package_data={
        'mlrank': ['tc.dat', 'eda.dat'],
    },
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.5",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 1 - Planning",
    ],
)
